
#include <jni.h>
#include <assert.h>
#include <dirent.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h> 
#include <stdio.h>

#include <android/log.h>

#include "tmodule.h"


#ifdef __cplusplus
extern "C" {
#endif


#include "hpnsPlatform.h"

extern int hpnsProcessDnsResult(UINT32 ip);

#ifdef __cplusplus
}
#endif

typedef union {
    JNIEnv* env;
    void* venv;
} UnionJNIEnvToVoid;

UnionJNIEnvToVoid g_uenv;
JNIEnv* g_jniEnv = NULL;
jclass g_classAdp = NULL;
jclass g_regInfoClass = NULL;
JavaVM* g_JavaVm = NULL;


#define EngineAdapterLog(...) ((void)__android_log_print(ANDROID_LOG_INFO, "HpnsEngineAdapter", __VA_ARGS__))

extern "C" {
 extern int hpnsSendAuthReqToEngine(int appid, char* paccountid);
 extern int hpnsSendUnReqToEngine(int appid, char* paccountid);
 extern void nprintf (char *format,...);
 extern int nmSystemInit();
 extern int nmsSystemDestroy();
 extern int channelId;
 extern int hpnsApiQueryAppViaAppId(UINT32 appId, char regId[], UINT32 regIdLen);
 extern int hpnsApiGetConfigInfo(short *majorVersion, short *minorVersion, int *connStatus,  char hidstr[ ]);
 extern void catchSystemLog();
 extern void hpnsSendNetworkStatusChangedReqToEngine(int flag); 
 extern void hpnsSendTimerMsgToEngine(int mid);
 extern int hpnsGetServerIP();
 extern int hpnsGetServerPort();
 extern int hpnsGetSessionId();
 extern int hpnsGetLocalSessionId();
 extern int hpnsGetStatus();
 extern unsigned char* hpnsGetHID();
 extern int hpnsCheckUpgrade();
 extern int hpnsGetMajorVersion();
 extern int hpnsGetMinorVersion();
 extern int hpnsCloseLog();
 extern int hpnsInitLog() ;
 extern int hpnsRestPushServerInfo(char *domain, char *newIp, int startPort, int portNum, int tcpPort);
 extern void hpnsGetServiceIp(char hidStr[]);
 extern void  hpnsGetServiceDns(char dnsStr[]);
}



char* jstring2char(JNIEnv *env, jstring pName)
{
  	jsize size = env->GetStringLength(pName);
	char * str = (char *)malloc(size + 1);
	if(NULL != pName){
		const char * jName = env->GetStringUTFChars(pName, NULL);
		memset(str, 0, size + 1);
		memcpy((void *)str, (void *)jName, size);
		env->ReleaseStringUTFChars(pName, jName);
	}
	return str;
} 



void proxy_nmSystemInit(JNIEnv *env, jobject thiz)
{
	nmSystemInit();
}

void proxy_nmsSystemDestroy(JNIEnv *env, jobject thiz)
{
	nmsSystemDestroy();
}

void proxy_nmsSendAuthReqToEngine(JNIEnv *env, jobject thiz, jint jappid, jstring jaccountid)
{
	//SHpnsAppProfile appprofile = {0};
	char* paccountid = jstring2char(env, jaccountid);
	//strncpy((char*)appprofile.senderId, paccountid, sizeof(appprofile.senderId));
	//EngineAdapterLog((char*)"appId:%d  senderId:%s\n", appprofile.appId, appprofile.senderId );
	hpnsSendAuthReqToEngine(jappid, paccountid);
	free(paccountid);
	//env->DeleteLocalRef(jaccountid);
	
	//hpnsSendMsgToEngine(HPNS_PE_MSG_REG_REQ, (UINT8 *)(&appprofile), sizeof(SHpnsAppProfile));
}

void proxy_nmsSendUnReqToEngine(JNIEnv *env, jobject thiz, jint jappid, jstring jaccountid)
{
	//SHpnsAppProfile appprofile = {0};
	char* paccountid = jstring2char(env, jaccountid);
	//strncpy((char*)appprofile.senderId, paccountid, sizeof(appprofile.senderId));
	//EngineAdapterLog((char*)"appId:%d  senderId:%s\n", appprofile.appId, appprofile.senderId );
	hpnsSendUnReqToEngine(jappid, paccountid);
	free(paccountid);
	//env->DeleteLocalRef(jaccountid);
	
	//hpnsSendMsgToEngine(HPNS_PE_MSG_REG_REQ, (UINT8 *)(&appprofile), sizeof(SHpnsAppProfile));
}

void proxy_nmsProcessNetworkStatechanged(JNIEnv *env, jobject thiz, jint flag)
{
	//hpnsSendMsgToEngine(HPNS_MSG_NETWORK_STATE_CHANGED, (UINT
	hpnsSendNetworkStatusChangedReqToEngine(flag);
}


void proxy_nmsProcessRestPushSeverInfo(JNIEnv *env, jobject thiz, jstring jnewDomain, jstring jnewIp, jint startPort, jint portNum, jint tcpPort)
{
	char *pDomain = jstring2char(env, jnewDomain);
	char *pnewIp  = jstring2char(env, jnewIp);
	
	hpnsRestPushServerInfo(pDomain, pnewIp, startPort, portNum, tcpPort);
	if(pDomain)
		free(pDomain);
	if(pnewIp)
		free(pnewIp);
}

void proxy_nmsSendTimerMsgToEngine(JNIEnv *env, jobject thiz, jint mid)
{
         hpnsSendTimerMsgToEngine(mid);
}


void proxy_nmsSendMsgToEngine(JNIEnv *env, jobject thiz, jint id, jstring msg)
{
}

int proxy_nmsGetchannelId(JNIEnv *env, jobject thiz)
{
	return channelId;
}

int proxy_nmsGetServerIP(JNIEnv *env, jobject thiz)
{
	return hpnsGetServerIP();
}

int proxy_nmsGetServerPort(JNIEnv *env, jobject thiz)
{
	return hpnsGetServerPort();
}

int proxy_nmsGetSessionId(JNIEnv *env, jobject thiz)
{
	return hpnsGetSessionId();
}

int proxy_nmsGetLocalSessionId(JNIEnv *env, jobject thiz)
{
	return hpnsGetLocalSessionId();
}

int proxy_nmsGetStatus(JNIEnv *env, jobject thiz)
{
	return hpnsGetStatus();
}

int proxy_nmsCheckUpgrade(JNIEnv *env, jobject thiz)
{
	return hpnsCheckUpgrade();
}

int proxy_nmsGetMajorVersion(JNIEnv *env, jobject thiz)
{
	return hpnsGetMajorVersion();
}


int proxy_nmsGetMinorVersion(JNIEnv *env, jobject thiz)
{
	return hpnsGetMinorVersion();
}

jstring proxy_nmsGetHID(JNIEnv *env, jobject thiz)
{
	#define HPNS_HID_LEN            8
	int i;
	jstring str;
	char hidStr[100] = {0};
	UINT8* hid = hpnsGetHID();
	sprintf(hidStr,"%d%d%d%d%d%d%d%d",*(hid+0),*(hid+1),*(hid+2),*(hid+3),*(hid+4),*(hid+5),*(hid+6),*(hid+7));
		
	nprintf("get hid form engine:%s",hidStr);
	str = env->NewStringUTF((char*)hidStr);
	return str;
}

jstring proxy_nmsGetVersion(JNIEnv *env, jobject thiz)
{
	jstring str;
	short majorVersion = 0;
	short minorVersion = 0;
	int connStatus = 0;
	int softwareupdate = 0;
	char hidstr[100]={0};
	hpnsApiGetConfigInfo(&majorVersion, &minorVersion, &connStatus, hidstr); 
	char path[255] = {0};
        snprintf(path, 255,"%d.%d-%s", majorVersion, minorVersion, hidstr);
	str = env->NewStringUTF((char*)path);
	return str;
}

jstring proxy_nmsQueryRegIdViaAppId(JNIEnv *env, jobject thiz, jint appId)
{
    int res = 0;
	char regId[HPNS_REGID_LEN*2+2] = {0};
	if (appId <= 0) {
		return NULL;
	}

	res = hpnsApiQueryAppViaAppId(appId, regId, HPNS_REGID_LEN*2);
	if (res == 0) {
		return env->NewStringUTF(regId);
	} else {
		return env->NewStringUTF("") ;
	}
}

static jstring proxy_nmsGetDNSHost(JNIEnv *env, jobject thiz) 
{
    char dns[HPNS_MAX_DOMAIN_LEN+1] = {0};
	hpnsGetDNSHost(dns) ;

    if (dns[0] == 0) 
        return env->NewStringUTF("") ;

    return env->NewStringUTF(dns) ;
}

static void proxy_nmsProcessDnsResult(JNIEnv *env, jobject thiz, jint ip) 
{
    hpnsProcessDnsResult(ip) ;
}

static void proxy_nmsCloseLog(JNIEnv *env, jobject thiz)
{
	hpnsCloseLog();
}

static void proxy_nmsInitLog(JNIEnv *env, jobject thiz)
{
	hpnsInitLog();
}

jstring proxy_nmsGetServiceIp(JNIEnv *env, jobject thiz)
{
	jstring str;
	char hidStr[100] = {0};
	
	hpnsGetServiceIp(hidStr);
	nprintf("hst00043 C put service ip:%s to java",(char*)hidStr);
	str = env->NewStringUTF((char*)hidStr);
	return str;
}
jstring proxy_nmsGetServiceDns(JNIEnv *env, jobject thiz)
{
	jstring str;
	char dnsStr[HPNS_MAX_DOMAIN_LEN] = {0};
	
	hpnsGetServiceDns(dnsStr);
	nprintf("hst00043 C put service dns:%s to java",(char*)dnsStr);
	str = env->NewStringUTF((char*)dnsStr);
	return str;
}

static JNINativeMethod methods[] = {
     /* name, signature, funcPtr */
	 /* msg api proxy*/
    {"nmSystemInit", "()V", (void*)&proxy_nmSystemInit},
    {"nmsSystemDestroy", "()V", (void*)&proxy_nmsSystemDestroy},
    {"nmsSendAuthReqToEngine", "(ILjava/lang/String;)V", (void *)&proxy_nmsSendAuthReqToEngine},
    {"nmsSendUnReqToEngine", "(ILjava/lang/String;)V", (void *)&proxy_nmsSendUnReqToEngine},
    {"nmsSendMsgToEngine", "(ILjava/lang/String;)V", (void*)&proxy_nmsSendMsgToEngine},
    {"nmsGetchannelId", "()I", (void*)&proxy_nmsGetchannelId},
    {"nmsGetVersion", "()Ljava/lang/String;", (void*)&proxy_nmsGetVersion},
    {"nmsProcessNetworkStatechanged", "(I)V", (void*)&proxy_nmsProcessNetworkStatechanged},
    {"nmsSendTimerMsgToEngine", "(I)V", (void*)&proxy_nmsSendTimerMsgToEngine},
    {"nmsGetServerIP", "()I", (void*)&proxy_nmsGetServerIP},
    {"nmsGetServerPort", "()I", (void*)&proxy_nmsGetServerPort},
    {"nmsGetSessionId", "()I", (void*)&proxy_nmsGetSessionId},
    {"nmsGetLocalSessionId", "()I", (void*)&proxy_nmsGetLocalSessionId}, 
    {"nmsGetStatus", "()I", (void*)&proxy_nmsGetStatus}, 
    {"nmsGetHID", "()Ljava/lang/String;", (void*)&proxy_nmsGetHID}, 
    {"nmsCheckUpgrade", "()I", (void*)&proxy_nmsCheckUpgrade}, 
    {"nmsGetMajorVersion", "()I", (void*)&proxy_nmsGetMajorVersion}, 
    {"nmsGetMinorVersion", "()I", (void*)&proxy_nmsGetMinorVersion},
    {"nmsQueryRegIdViaAppId", "(I)Ljava/lang/String;", (void*)&proxy_nmsQueryRegIdViaAppId},
    {"nmsGetDNSHost", "()Ljava/lang/String;", (void*)&proxy_nmsGetDNSHost}, 
    {"nmsProcessDnsResult", "(I)V", (void*)&proxy_nmsProcessDnsResult},  
    {"nmsCloseLog", "()V", (void*)&proxy_nmsCloseLog},
    {"nmsInitLog", "()V", (void*)&proxy_nmsInitLog},
    {"nmsRestPushSeverInfo", "(Ljava/lang/String;Ljava/lang/String;III)V", (void*)&proxy_nmsProcessRestPushSeverInfo},
	{"nmsGetServiceIp", "()Ljava/lang/String;", (void*)&proxy_nmsGetServiceIp}, 
	{"nmsGetServiceDns", "()Ljava/lang/String;", (void*)&proxy_nmsGetServiceDns}, 
};

/*
 * Register several native methods for one class.
 */
static int registerNativeMethods(JNIEnv* env, const char* className,
    JNINativeMethod* gMethods, int numMethods)
{
    jclass clazz = env->FindClass(className);

    if (clazz == NULL) {
        //nprintf((char *)"Native registration unable to find class %s", className);
        return JNI_FALSE;
    }

    if (env->RegisterNatives(clazz, gMethods, numMethods) < 0) {
        //nprintf((char *)"RegisterNatives failed for %s", className);
		if (env->ExceptionOccurred())        
		{    
			env->ExceptionDescribe();
		}
        return JNI_FALSE;
    }

    return JNI_TRUE;
}


static struct sigaction old_sa[NSIG];

void android_sigaction(int signum, siginfo_t *info, void *reserved){ 
	catchSystemLog(); 
	sleep(3);
	old_sa[signum].sa_handler(signum);
} 

void android_sigterm(int signum, siginfo_t *info, void *reserved){ 
	//catchSystemLog(); 
	nprintf((char*)"exiting....");
	sleep(3);
	old_sa[signum].sa_handler(signum);
} 

jint JNI_OnLoad(JavaVM* vm, void* reserved)
{
	if( NULL != g_JavaVm )
	{
		nprintf((char*)"JNI_OnLoad, javavm exists, no need to reload");
		return (jint)JNI_VERSION_1_2;
	}
		
    g_uenv.venv = NULL;
	g_JavaVm  = vm;

    if (vm->GetEnv(&g_uenv.venv, JNI_VERSION_1_2) != JNI_OK) {
        nprintf((char*)"JNI_OnLoad, GetEnv failed\n");
        return (jint)-1;
    }
    g_jniEnv = g_uenv.env;
    assert(g_jniEnv != NULL);

	jclass classAdp = g_jniEnv->FindClass("com/hissage/hpe/jni/HpnsPlatformAdapter");   
	g_classAdp = (jclass)g_jniEnv->NewGlobalRef(classAdp);
	jclass regInfoClass = g_jniEnv->FindClass("com/hissage/hpe/struct/SHpnsRegInfo");
	g_regInfoClass = (jclass)g_jniEnv->NewGlobalRef(regInfoClass);

    //nprintf((char *)"In JNI_OnLoad, javavm:%x, jnienv:%x\n", (void*)&g_JavaVm, (void*)&g_jniEnv);

    if (JNI_FALSE ==
        registerNativeMethods(g_jniEnv, 
                              "com/hissage/hpe/jni/HpnsEngineAdapter",
                              methods,
                              sizeof(methods) / sizeof(methods[0]))) {
        nprintf((char *)"native registration failed\n");
        return (jint)-1;
    }


    /* success -- return valid version number */
    nprintf((char *)"native registration succeeded!\n");
	g_jniEnv->DeleteLocalRef(classAdp);
	g_jniEnv->DeleteLocalRef(regInfoClass);
	struct sigaction handler = {0};	
	handler.sa_sigaction = android_sigaction;
	handler.sa_flags = SA_RESETHAND;
	#define CATCHSIG(X) sigaction(X, &handler, &old_sa[X])	
	CATCHSIG(SIGILL);	
	CATCHSIG(SIGABRT);	
	CATCHSIG(SIGBUS);
	CATCHSIG(SIGFPE);	
	CATCHSIG(SIGSEGV);	
	CATCHSIG(SIGSTKFLT);
	CATCHSIG(SIGPIPE);
	struct sigaction handler_sigterm = {0};	
	handler_sigterm.sa_sigaction = android_sigterm;
	handler_sigterm.sa_flags = SA_RESETHAND;
	sigaction(SIGTERM, &handler_sigterm, &old_sa[SIGTERM]);
	sigaction(SIGKILL, &handler_sigterm, &old_sa[SIGKILL]);

    //   nmSystemInit() ; 
       
    return (jint)JNI_VERSION_1_2;}
